// IUPACNucleotides.java
//
// (c) 1999-2001 PAL Development Core Team
//
// This package may be distributed under the
// terms of the Lesser GNU General Public License (LGPL)


package pal.datatype;


/**
 * implements DataType for nucleotides with ambiguous characters
 *
 * @version $Id: IUPACNucleotides.java,v 1.9 2001/11/26 03:58:59 matt Exp $
 *
 * @author Alexei Drummond
 */
public class IUPACNucleotides implements DataType
{

	private static final char[] CONVERSION_TABLE= { 'A','C','G','U','K','M','R','S','W','Y','B','D','H','V','N',UNKNOWN_CHARACTER};
	/**
	 * Get number of states.
	 */
	public int getNumStates()
	{
		return 15;
	}

	/**
	 * @retrun true if this state is an unknown state
	 */
	public boolean isUnknownState(int state) {
		return(state>=getNumStates())||(state<0);
	}

	/**
	 * returns the number of true non-ambiguous states.
	 */
	public int getNumSimpleStates() {
		return 4;
	}

	// Get state corresponding to character c
	public final int getState(char c) {

		switch (c) {
			case 'A': return  0;
			case 'C': return  1;
			case 'G': return  2;
			case 'T': return  3;
			case 'U': return  3;
			case 'K': return  4;
			case 'M': return  5;
			case 'R': return  6;
			case 'S': return  7;
			case 'W': return  8;
			case 'Y': return  9;
			case 'B': return 10;
			case 'D': return 11;
			case 'H': return 12;
			case 'V': return 13;
			case 'N': return 14;
			default: return  15;
		}
	}
	// Get character corresponding to a given state
	public char getChar(int state) {
		if(state<CONVERSION_TABLE.length&&state>=0){
			return CONVERSION_TABLE[state];
		}
		return UNKNOWN_CHARACTER;
	}

	// String describing the data type
	public String getDescription()
	{
		return "IUPACNucleotide";
	}

	// Get numerical code describing the data type
	public int getTypeID()
	{
		return DataType.IUPACNUCLEOTIDES;
	}

	/**
	 * returns true if this state is an ambiguous state.
	 */
	public boolean isAmbiguousState(int state) {
		if ((state > 3) && (state < getNumStates())) {
			return true;
		}
		return false;
	}

	/**
	 * returns an array containing the non-ambiguous states
	 * that this state represents.
	 */
	public int[] getSimpleStates(int state) {

		String stateString = "";

		switch (state) {
			case 0: stateString = "A"; break;
			case 1: stateString = "C"; break;
			case 2: stateString = "G"; break;
			case 3: stateString = "T"; break;
			case 4: stateString = "GT"; break;
			case 5: stateString = "AC"; break;
			case 6: stateString = "AG"; break;
			case 7: stateString = "CG"; break;
			case 8: stateString = "AT"; break;
			case 9: stateString = "CT"; break;
			case 10: stateString = "CGT"; break;
			case 11: stateString = "AGT"; break;
			case 12: stateString = "ACT"; break;
			case 13: stateString = "ACG"; break;
			case 14: stateString = "ACGT"; break;
			default: stateString = "ACGT"; break;
		}

		int[] states = new int[stateString.length()];
		for (int i = 0; i < stateString.length(); i++) {
			states[i] = getState(stateString.charAt(i));
		}

		return states;
	}


	/**
	 * @return true if this state is a gap
	 */
	public boolean isUnknownChar(char c) {
		return isUnknownState(getChar(c));
	}
}
