// NucleotideData.java
//
// (c) 1999-2001 PAL Development Core Team
//
// This package may be distributed under the
// terms of the Lesser GNU General Public License (LGPL)


package pal.datatype;


/**
 * implements DataType for nucleotides
 *
 * @version $Id: Nucleotides.java,v 1.12 2001/11/26 03:58:59 matt Exp $
 *
 * @author Korbinian Strimmer
 * @author Alexei Drummond
 */
public class Nucleotides extends SimpleDataType {

	public static final int A_STATE = 0;
	public static final int C_STATE = 1;
	public static final int G_STATE = 2;
	public static final int UT_STATE = 3;

	private static final long serialVersionUID=-497943046234232204L;

	//
	// Variables
	//

	boolean isRNA_;
	private static final char[] DNA_CONVERASION_TABLE = {'A', 'C', 'G', 'T', UNKNOWN_CHARACTER};
	private static final char[] RNA_CONVERASION_TABLE = {'A', 'C', 'G', 'T', UNKNOWN_CHARACTER};

	//Must stay after static CONVERSION_TABLE stuff!
	public static final Nucleotides DEFAULT_INSTANCE = new Nucleotides();
	//For faster conversion!
	char[] conversionTable_ = DNA_CONVERASION_TABLE;
	//
	// Serialization code
	//

	//serialver -classpath ./classes cebl.ceblet.AbstractCeblet
	private void writeObject(java.io.ObjectOutputStream out) throws java.io.IOException {
		out.writeByte(1); //Version number
		out.writeBoolean(isRNA_);
	}

	private void readObject(java.io.ObjectInputStream in) throws java.io.IOException, ClassNotFoundException{
		byte version = in.readByte();
		switch(version) {
			default : {
				 isRNA_ = in.readBoolean();
				 conversionTable_ = (isRNA_ ? RNA_CONVERASION_TABLE : DNA_CONVERASION_TABLE );
				 break;
			}
		}
	}


	public Nucleotides() {
		this(false);
	}

	/** If isRNA is true than getChar(state) will return a U instead of a T */
	public Nucleotides(boolean isRNA) {
		this.isRNA_ = isRNA;
		conversionTable_ = (isRNA_ ? RNA_CONVERASION_TABLE : DNA_CONVERASION_TABLE );
		//The following is to make absolutely sure RNA_CONVERSION_TABLE and co is initialised.
		//The problem results from the DEFAULT_INSTANCE constructor and the order of evaluation for
		// the static variables.
		if(conversionTable_==null) {
			char[] defaultTable = {'A', 'C', 'G', 'T', UNKNOWN_CHARACTER};
			if(isRNA_) {
				defaultTable[0] = 'U';
			}
			conversionTable_ = defaultTable;
		}

	}

	// Get number of bases
	public int getNumStates()
	{
		return 4;
	}

	/**
	 * @return true if this state is an unknown state
	 */
	public boolean isUnknownState(int state) {
		return(state>=4)||(state<0);
	}

	/**
	 * Get state corresponding to character c <BR>
	 * <B>NOTE</B>: IF YOU CHANGE THIS IT MAY STOP THE NUCLEOTIDE TRANSLATOR FROM WORKING!
	 * - It relies on the fact that all the states for 'ACGTU' are between [0, 3]
	 */
	public int getState(char c)
	{
		switch (c)
		{
			case 'A':
				return A_STATE;
			case 'C':
				return C_STATE;
			case 'G':
				return G_STATE;
			case 'T':
				return UT_STATE;
			case 'U':
				return UT_STATE;

			case UNKNOWN_CHARACTER:
				return 4;

			default:
				return 4;
		}
	}

	/**
	 * Get character corresponding to a given state
	 */
	public char getChar(int state) {
		if(state<conversionTable_.length&&state>=0){
			return conversionTable_[state];
		}
		return UNKNOWN_CHARACTER;
	}

	/**
	 * @return a string describing the data type
	 */
	public String getDescription()	{
		return NUCLEOTIDE_DESCRIPTION;
	}

	/**
	 * @return the unique numerical code describing the data type
	 */
	public int getTypeID() {
		return 0;
	}

	/**
	 * @return true if A->G, G->A, C->T, or T->C
	 * if firstState equals secondState returns FALSE!
	 */
	public final boolean isTransistionByState(int firstState, int secondState) {
		switch(firstState) {
			case A_STATE: {
				if(secondState==G_STATE) {
					return true;
				}
				return false;
			}
			case C_STATE : {
				if(secondState==UT_STATE) {
					return true;
				}
				return false;
			}
			case G_STATE : {
				if(secondState==A_STATE) {
					return true;
				}
				return false;
			}
			case UT_STATE : {
				if(secondState==C_STATE) {
					return true;
				}
				return false;
			}
		}
		return false;
	}

	/**
	 * @return true if A->G, G->A, C->T, or T->C
	 * if firstState equals secondState returns FALSE!
	 * (I've renamed things to avoid confusion between java typing of ints and chars)
	*/
	public final boolean isTransistionByChar(char firstChar, char secondChar) {
		//I'm leaving it open to a possible optimisation if anyone cares.
		return isTransistionByState(getState(firstChar), getState(secondChar));
	}
}
