// SimulatedAlignment.java
//
// (c) 1999-2001 PAL Development Core Team
//
// This package may be distributed under the
// terms of the Lesser GNU General Public License (LGPL)


package pal.tree;

import pal.datatype.*;
import pal.substmodel.*;
import pal.alignment.*;
import pal.math.*;
import pal.misc.*;


/**
 * generates an artificial data set
 *
 * @version $Id: SimulatedAlignment.java,v 1.14 2001/11/20 07:35:44 matt Exp $
 *
 * @author Korbinian Strimmer
 * @author Alexei Drummond
 */
public class SimulatedAlignment extends AbstractAlignment
{
	//
	// Public stuff
	//


	//
	// Private stuff
	//

	private Tree tree;
	private SubstitutionModel model;
	private double[] cumFreqs;
	private int[] rateAtSite;
	private double[] cumRateProbs;
	private int numStates;
	private byte[][] stateData;
	private MersenneTwisterFast rng;

		//
	// Serialization
	//


	//private static final long serialVersionUID = -5197800047652332969L;

	//serialver -classpath ./classes pal.tree.SimulatedAlignment
	private void writeObject(java.io.ObjectOutputStream out) throws java.io.IOException {
		out.writeByte(1); //Version number
		out.writeObject(tree);
		out.writeObject(model);
		out.writeObject(cumFreqs);
		out.writeObject(rateAtSite);
		out.writeObject(cumRateProbs);
		out.writeObject(stateData);
	}

	private void readObject(java.io.ObjectInputStream in) throws java.io.IOException, ClassNotFoundException{
		byte version = in.readByte();
		switch(version) {
			default : {
				tree = (Tree)in.readObject();
				model = (SubstitutionModel)in.readObject();
				cumFreqs = (double[])in.readObject();
				rateAtSite = (int[])in.readObject();
				cumRateProbs = (double[])in.readObject();
				stateData = (byte[][])in.readObject();
				numStates = dataType.getNumStates();
				rng = new MersenneTwisterFast();
				break;
			}
		}
	}

	/**
	 * Inititalisation
	 *
	 * @param sites number of sites
	 * @param t     tree relating the sequences
	 * @param m     model of evolution
	 */
	public SimulatedAlignment(int sites, Tree t, SubstitutionModel m) {
		rng = new MersenneTwisterFast();
		dataType = m.getRateMatrix().getDataType();
		numStates = dataType.getNumStates();
		model = m;

		tree = t;
		tree.createNodeList();

		numSeqs = tree.getExternalNodeCount();
		numSites = sites;
		idGroup = new SimpleIdGroup(numSeqs);

		for (int i = 0; i < numSeqs; i++)
		{
			idGroup.setIdentifier(i, tree.getExternalNode(i).getIdentifier());
		}

		stateData = new byte[numSeqs][numSites];

		for (int i = 0; i < tree.getExternalNodeCount(); i++)
		{
			tree.getExternalNode(i).setSequence(stateData[i]);
		}
		for (int i = 0; i < tree.getInternalNodeCount()-1; i++)
		{
			tree.getInternalNode(i).setSequence(new byte[numSites]);
		}


		rateAtSite = new int[numSites];
		cumFreqs = new double[numStates];
		cumRateProbs = new double[m.getRateDistribution().numRates];
	}


	// Implementation of abstract Alignment method

	/** sequence alignment at (sequence, site) */
	public char getData(int seq, int site)
	{
		return dataType.getChar(stateData[seq][site]);
	}


	/** generate new artificial data set (random root sequence) */
	public void simulate() {
		simulate(makeRandomRootSequence());
	}

	/** generate new artificial data set (random root sequence) */
	public void simulate(String givenRootSequence)	{
		simulate(DataTypeUtils.getByteStates(givenRootSequence, model.getRateMatrix().getDataType()));
	}
	/** generate new artificial data set (specified root sequence) */
	public void simulate(byte[] rootSeq)
	{
		// Check root sequence
		for (int i = 0; i < numSites; i++)
		{
			if (rootSeq[i] >= numStates || rootSeq[i] < 0)
			{
				throw new IllegalArgumentException("Root sequence contains illegal state (?,-, etc.)");
			}
		}

		tree.getInternalNode(tree.getInternalNodeCount()-1).setSequence(rootSeq);

		// Assign new rate categories
		assignRates();

		// Visit all nodes except root
		Node node = NodeUtils.preorderSuccessor(tree.getRoot());
		do
		{
			determineMutatedSequence(node);
			node = NodeUtils.preorderSuccessor(node);
		}
		while (node != tree.getRoot());
	}

	private void determineMutatedSequence(Node node)
	{
		if (node.isRoot()) throw new IllegalArgumentException("Root node not allowed");

		model.setDistance(node.getBranchLength());

		byte[] oldS = node.getParent().getSequence();
		byte[] newS = node.getSequence();

		for (int i = 0; i < numSites; i++)
		{
			cumFreqs[0] = model.transProb(rateAtSite[i], oldS[i], 0);
			for (int j = 1; j < numStates; j++)
			{
				cumFreqs[j] = cumFreqs[j-1] + model.transProb(rateAtSite[i], oldS[i], j);
			}

			newS[i] = (byte) randomChoice(cumFreqs);
		}
	}

	private byte[] makeRandomRootSequence()
	{
		double[] frequencies = model.getRateMatrix().getEquilibriumFrequencies();
		cumFreqs[0] = frequencies[0];
		for (int i = 1; i < numStates; i++)	{
			cumFreqs[i] = cumFreqs[i-1] + frequencies[i];
		}
		byte[] rootSequence = new byte[numSites];
		for (int i = 0; i < numSites; i++)
		{
			rootSequence[i] = (byte) randomChoice(cumFreqs);
		}
		return rootSequence;
	}

	private void assignRates()
	{
		cumRateProbs[0] = model.getRateDistribution().probability[0];
		for (int i = 1; i < model.getRateDistribution().numRates; i++)
		{
			cumRateProbs[i] = cumRateProbs[i-1] + model.getRateDistribution().probability[i];
		}

		for (int i = 0; i < numSites; i++)
		{
			rateAtSite[i] = randomChoice(cumRateProbs);
		}


	}

	// Chooses one category if a cumulative probability distribution is given
	private int randomChoice(double[] cf)
	{

		double rnd = rng.nextDouble();

		int s;
		if (rnd <= cf[0])
		{
			s = 0;
		}
		else
		{
			for (s = 1; s < cf.length; s++)
			{
				if (rnd <= cf[s] && rnd > cf[s-1])
				{
					break;
				}
			}
		}

		return s;
	}
}
